@@ -33,14 +33,17 @@
 #include <gtk/gtk.h>
 #include <glade/glade.h>
 #include <gconf/gconf-client.h>
-#include <libgnome/libgnome.h>
-#include <libgnomeui/libgnomeui.h>
 #include <dbus/dbus-glib.h>
+#include <glib/gi18n.h>
 
 #ifdef VINO_ENABLE_KEYRING
 #include <gnome-keyring.h>
 #endif
 
+#ifdef VINO_ENABLE_LIBUNIQUE
+#include <unique/unique.h>
+#endif
+
 #define VINO_PREFS_DIR                    "/desktop/gnome/remote_access"
 #define VINO_PREFS_ENABLED                VINO_PREFS_DIR "/enabled"
 #define VINO_PREFS_PROMPT_ENABLED         VINO_PREFS_DIR "/prompt_enabled"
@@ -54,8 +57,9 @@
 #define VINO_PREFS_USE_ALTERNATIVE_PORT   VINO_PREFS_DIR "/use_alternative_port"
 #define VINO_PREFS_ALTERNATIVE_PORT       VINO_PREFS_DIR "/alternative_port"
 #define VINO_PREFS_LOCK_SCREEN            VINO_PREFS_DIR "/lock_screen_on_disconnect"
+#define VINO_PREFS_DISABLE_BACKGROUND     VINO_PREFS_DIR "/disable_background"
 
-#define N_LISTENERS 12
+#define N_LISTENERS 13
 
 #define VINO_DBUS_BUS_NAME  "org.gnome.Vino"
 #define VINO_DBUS_INTERFACE "org.gnome.VinoScreen"
@@ -83,7 +87,11 @@ typedef struct {
   GtkWidget   *use_alternative_port_toggle;
   GtkWidget   *alternative_port_entry;
   GtkWidget   *lock_screen_toggle;
+  GtkWidget   *disable_background_toggle;
   GtkTooltips *tips;
+#ifdef VINO_ENABLE_LIBUNIQUE
+  UniqueApp   *app;
+#endif
 
   guint        listeners [N_LISTENERS];
   int          n_listeners;
@@ -176,6 +184,7 @@ vino_preferences_dialog_update_for_allowed (VinoPreferencesDialog *dialog,
   gtk_widget_set_sensitive (dialog->alternative_port_entry,      allowed &&
 			    gconf_client_get_bool (dialog->client, VINO_PREFS_USE_ALTERNATIVE_PORT, NULL));
   gtk_widget_set_sensitive (dialog->lock_screen_toggle,          allowed);
+  gtk_widget_set_sensitive (dialog->disable_background_toggle,   allowed);
 }
 
 static void
@@ -1002,6 +1011,67 @@ vino_preferences_dialog_setup_password_entry (VinoPreferencesDialog *dialog)
     }
 }
 
+static void
+vino_preferences_dialog_disable_background_toggled (GtkToggleButton       *toggle,
+                                                    VinoPreferencesDialog *dialog)
+{
+  gboolean disable_background;
+
+  disable_background = gtk_toggle_button_get_active (toggle);
+
+  gconf_client_set_bool (dialog->client, VINO_PREFS_DISABLE_BACKGROUND, disable_background, NULL);
+}
+
+static void
+vino_preferences_dialog_disable_background_notify (GConfClient           *client,
+                                                   guint                  cnx_id,
+                                                   GConfEntry            *entry,
+                                                   VinoPreferencesDialog *dialog)
+{
+  gboolean disable_background;
+
+  if (!entry->value || entry->value->type != GCONF_VALUE_BOOL)
+    return;
+
+  disable_background = gconf_value_get_bool (entry->value) != FALSE;
+
+  if (disable_background != gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (dialog->disable_background_toggle)))
+    {
+      gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (dialog->disable_background_toggle), disable_background);
+    }
+}
+
+static gboolean
+vino_preferences_dialog_setup_disable_background_toggle (VinoPreferencesDialog *dialog)
+{
+  gboolean disable_background;
+
+  dialog->disable_background_toggle = glade_xml_get_widget (dialog->xml, "disable_background_toggle");
+  g_assert (dialog->disable_background_toggle != NULL);
+
+  disable_background = gconf_client_get_bool (dialog->client, VINO_PREFS_DISABLE_BACKGROUND, NULL);
+
+  gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (dialog->disable_background_toggle), disable_background);
+
+  g_signal_connect (dialog->disable_background_toggle, "toggled",
+		    G_CALLBACK (vino_preferences_dialog_disable_background_toggled), dialog);
+
+  if (!gconf_client_key_is_writable (dialog->client, VINO_PREFS_DISABLE_BACKGROUND, NULL))
+    {
+      gtk_widget_set_sensitive (dialog->disable_background_toggle, FALSE);
+      gtk_widget_show (dialog->writability_warning);
+    }
+
+  dialog->listeners [dialog->n_listeners] = 
+    gconf_client_notify_add (dialog->client,
+			     VINO_PREFS_DISABLE_BACKGROUND,
+			     (GConfClientNotifyFunc) vino_preferences_dialog_disable_background_notify,
+			     dialog, NULL, NULL);
+  dialog->n_listeners++;
+
+  return disable_background;
+}
+
 static void
 vino_preferences_server_updated (DBusGProxy *proxy,
                                  const char *name,
@@ -1276,7 +1346,7 @@ vino_preferences_dialog_uri_hook (GtkLinkButton *button,
   screen = gtk_widget_get_screen (GTK_WIDGET (button));
 
   error = NULL;
-  if (!gnome_url_show_on_screen (link, screen, &error))
+  if (!gtk_show_uri (screen, link, GDK_CURRENT_TIME, &error))
     {
       /* FIXME better error handling!
        *       What best to do? For the specific case
@@ -1343,7 +1413,8 @@ vino_preferences_dialog_response (GtkWidget             *widget,
 				  int                    response,
 				  VinoPreferencesDialog *dialog)
 {
-  GError *error;
+  GError    *error;
+  GdkScreen *screen;
 
   if (response != GTK_RESPONSE_HELP)
     {
@@ -1351,8 +1422,10 @@ vino_preferences_dialog_response (GtkWidget             *widget,
       return;
     }
 
+  screen = gtk_widget_get_screen (widget);
   error = NULL;
-  gnome_help_display_desktop (NULL, "user-guide", "user-guide.xml", "goscustdesk-90", &error);
+
+  gtk_show_uri (screen, "ghelp:user-guide?goscustdesk-90", GDK_CURRENT_TIME, &error);
   if (error)
     {
       GtkWidget *message_dialog;
@@ -1436,6 +1509,7 @@ vino_preferences_dialog_init (VinoPreferencesDialog *dialog)
   vino_preferences_dialog_setup_alternative_port_entry      (dialog);
   vino_preferences_dialog_setup_use_alternative_port_toggle (dialog);
   vino_preferences_dialog_setup_lock_screen_toggle          (dialog);
+  vino_preferences_dialog_setup_disable_background_toggle   (dialog);
 
   g_assert (dialog->n_listeners == dialog->expected_listeners);
 
@@ -1443,6 +1517,10 @@ vino_preferences_dialog_init (VinoPreferencesDialog *dialog)
 
   gtk_widget_show (dialog->dialog);
 
+#ifdef VINO_ENABLE_LIBUNIQUE
+  unique_app_watch_window (dialog->app, GTK_WINDOW (dialog->dialog));
+#endif
+
   return TRUE;
 
 #undef VINO_GLADE_FILE  
@@ -1484,6 +1562,36 @@ vino_preferences_dialog_finalize (VinoPreferencesDialog *dialog)
   if (dialog->xml)
     g_object_unref (dialog->xml);
   dialog->xml = NULL;
+
+#ifdef VINO_ENABLE_LIBUNIQUE
+  if (dialog->app)
+    g_object_unref (dialog->app);
+  dialog->app = NULL;
+#endif
+}
+
+static gboolean
+vino_preferences_is_running (VinoPreferencesDialog *dialog)
+{
+#ifdef VINO_ENABLE_LIBUNIQUE
+  dialog->app = unique_app_new ("org.gnome.Vino.Preferences", NULL);
+
+  if (unique_app_is_running (dialog->app))
+    {
+      UniqueResponse response;
+
+      response = unique_app_send_message (dialog->app, UNIQUE_ACTIVATE, NULL);
+        
+      g_object_unref (dialog->app);
+      dialog->app = NULL;
+       
+      return response == UNIQUE_RESPONSE_OK;
+    }
+  else
+    return FALSE;
+#else
+  return FALSE;
+#endif
 }
 
 int
@@ -1495,9 +1603,11 @@ main (int argc, char **argv)
   bind_textdomain_codeset (GETTEXT_PACKAGE, "UTF-8");
   textdomain (GETTEXT_PACKAGE);
 
-  gnome_program_init (PACKAGE, VERSION, LIBGNOMEUI_MODULE,
-		      argc, argv, NULL);
+  gtk_init (&argc, &argv);
 
+  if (vino_preferences_is_running (&dialog))
+    return 0;
+  
   if (!vino_preferences_dialog_init (&dialog))
     {
       vino_preferences_dialog_finalize (&dialog);
